#include "src/windows-updater/download-dialog.hpp"

#include <QMessageBox>
#include "src/types.hpp"

#if QT_VERSION >= QT_VERSION_CHECK( 5, 15, 0 )
	#define ON_NETWORK_ERROR QOverload<QNetworkReply::NetworkError>::of( &QNetworkReply::errorOccurred )
#else
	#define ON_NETWORK_ERROR QOverload<QNetworkReply::NetworkError>::of( &QNetworkReply::error )
#endif

DownloadDialog::DownloadDialog( const QUrl &url, const fs::path &filePath ) : QProgressDialog() {
	m_networkManager = new QNetworkAccessManager();
	m_stream = m_networkManager->get( QNetworkRequest( url ) );

	fs::create_directories( filePath.parent_path() );
	m_file = new QFile( filePath.string().c_str() );
	m_file->open( QIODevice::WriteOnly | QIODevice::Truncate );

	setLabelText( "Downloading installer..." );
	setRange( 0, 1 );
	setValue( 0 );

	connect( m_stream, &QNetworkReply::readyRead, this, &DownloadDialog::dataReceived );
	connect( m_stream, &QNetworkReply::downloadProgress, this, &DownloadDialog::onDownloadProgress );
	connect( m_stream, ON_NETWORK_ERROR, this, &DownloadDialog::onDownloadFailed );
	connect( m_stream, &QNetworkReply::finished, this, &DownloadDialog::onDownloadFinished );
}

DownloadDialog::~DownloadDialog() {
	m_stream->deleteLater();
	m_networkManager->deleteLater();
	m_file->flush();
	m_file->close();
	m_file = nullptr;
	delete m_file;
}

void DownloadDialog::onDownloadProgress( qint64 downloaded, qint64 total ) {
	setMaximum( (int)total );
	setValue( (int)downloaded );
}

void DownloadDialog::onDownloadFailed( [[maybe_unused]] QNetworkReply::NetworkError error ) {
	QMessageBox::critical( nullptr, "Download Failed", "Failed to download the latest installer. Try again later." );
	reject();
}

void DownloadDialog::dataReceived() {
	if( m_file != nullptr ) {
		m_file->write( m_stream->readAll() );
	}
}

void DownloadDialog::onDownloadFinished() {
	if( m_stream->error() == QNetworkReply::NoError ) {
		accept();
	} else {
		reject();
	}
}
